@@ -8,6 +8,7 @@ from .compiler import (
     DEFAULT_TEAL_VERSION,
     CompileOptions,
     compileTeal,
+    OptimizeOptions,
 )
 from .types import TealType
 from .errors import TealInternalError, TealTypeError, TealInputError, TealCompileError
@@ -23,6 +24,7 @@ __all__ = (
         "DEFAULT_TEAL_VERSION",
         "CompileOptions",
         "compileTeal",
+        "OptimizeOptions",
         "TealType",
         "TealInternalError",
         "TealTypeError",
