@@ -2,15 +2,25 @@ import pytest
 
 from .. import *
 
+# this is not necessary but mypy complains if it's not included
+from .. import CompileOptions
+
+options = CompileOptions()
+
+
 def test_int():
     values = [0, 1, 8, 232323, 2**64 - 1]
 
     for value in values:
         expr = Int(value)
         assert expr.type_of() == TealType.uint64
-        assert expr.__teal__() == [
-            TealOp(Op.int, value)
-        ]
+
+        expected = TealSimpleBlock([TealOp(expr, Op.int, value)])
+
+        actual, _ = expr.__teal__(options)
+
+        assert actual == expected
+
 
 def test_int_invalid():
     with pytest.raises(TealInputError):
@@ -20,14 +30,18 @@ def test_int_invalid():
         Int(-1)
 
     with pytest.raises(TealInputError):
-        Int(2 ** 64)
-    
+        Int(2**64)
+
     with pytest.raises(TealInputError):
         Int("0")
 
+
 def test_enum_int():
     expr = EnumInt("OptIn")
     assert expr.type_of() == TealType.uint64
-    assert expr.__teal__() == [
-        TealOp(Op.int, "OptIn")
-    ]
+
+    expected = TealSimpleBlock([TealOp(expr, Op.int, "OptIn")])
+
+    actual, _ = expr.__teal__(options)
+
+    assert actual == expected
