@@ -1,33 +1,5 @@
-#!/usr/bin/env python3
-"""
-Helper functions and classes
-"""
+from .errors import TealInternalError
 
-import subprocess
-   
-label_count = 0
-
-def reset_label_count():
-    global label_count
-    label_count = 0
-
-def new_label():
-    global label_count
-    new_l = "l{}".format(label_count)
-    label_count += 1
-    return new_l
-
-def execute(args):
-    """ Execute in bash, return stdout and stderr in string
-    
-    Arguments:
-    args: command and arguments to run, e.g. ['ls', '-l']
-    """
-    process = subprocess.Popen(args, stdout=subprocess.PIPE,
-                               stderr=subprocess.PIPE)
-    stdout, stderr = process.communicate()
-    
-    return (stdout.decode("utf-8"), stderr.decode("utf-8"))
 
 def escapeStr(s: str) -> str:
     """Escape a UTF-8 string for use in TEAL assembly.
@@ -55,7 +27,34 @@ def escapeStr(s: str) -> str:
     s = s.encode("utf-8").decode("latin-1").encode("unicode-escape").decode("latin-1")
 
     # Escape double quote characters (not covered by unicode-escape) but leave in single quotes
-    s = s.replace("\"", "\\\"")
-    
+    s = s.replace('"', '\\"')
+
     # Surround string in double quotes
-    return "\"" + s + "\""
+    return '"' + s + '"'
+
+
+def unescapeStr(s: str) -> str:
+    if len(s) < 2 or s[0] != '"' or s[-1] != '"':
+        raise ValueError("Escaped string if of the wrong format")
+    s = s[1:-1]
+    s = s.replace('\\"', '"')
+    s = s.encode("latin-1").decode("unicode-escape").encode("latin-1").decode("utf-8")
+    return s
+
+
+def correctBase32Padding(s: str) -> str:
+    content = s.split("=")[0]
+    trailing = len(content) % 8
+
+    if trailing == 2:
+        content += "=" * 6
+    elif trailing == 4:
+        content += "=" * 4
+    elif trailing == 5:
+        content += "=" * 3
+    elif trailing == 7:
+        content += "="
+    elif trailing != 0:
+        raise TealInternalError("Invalid base32 content")
+
+    return content
